## Zinc Coordination Geometry Scripts

This set of scripts runs the full set of analyses used in:

Yao, S., Flight, R. M., Rouchka, E. C. and Moseley, H. N. B. (2015), A less-biased analysis of metalloproteins reveals novel zinc coordination geometries. Proteins. doi: 10.1002/prot.24834

## Installation

You should un-tar the downloaded file:

```
tar -xzpf zinc_analysis.tar.gz
```

This creates the directory *zincPaperAnalysis*, that contains a set of directories:

  * atom2seq: R package that does alignment and functional enrichment
  * categoryCompareANY: allows arbitrary enrichment calculations
  * mainScripts: the main scripts that do the analysis
  * output: where all of the outputs are saved
  * output_org: the outputs generated by the scripts by the authors
  * output_manuscript: the outputs that are used in the results of the manuscript
  * znProtein: supporting scripts

## Notes on Output_org and Output_manuscript

Output_org is the set of outputs generated by running the full analysis by the authors.
Every effort has been made to ensure that these results are very similar to the
results in the manuscript, however there are some differences, primarily the result
of different initialization of the clusters.

Output_manuscript is the set of results that were used to generate the figures / results
in the actual manuscript. A slightly different, **not fully scripted** workflow was
used to generate and process these results. Therefore some data is stored as text in
contrast to RData files, and cluster membership and identification between the two
are different. However, the conclusions from both sets of data are the same.

## Prerequisites

To run it, you will need 16GB of Ram, and also need to install (on Linux):

perl::Math::MatrixReal
perl::Statistics::Distributions
perl::Clone
perl::JSON (optional for certain settings)
perl::Data::Dumper::Concise (optional for certain settings)

Interproscan (https://code.google.com/p/interproscan/)

libxml2-devel (for compiling some R packages from source)

R
R-devel

And then the following R packages:

Biostrings
Biobase
AnnotationDbi
Category
annotate
clue
cluster
randomForest

In addition, two custom packages also need to be installed, which are included. Two install those, the easiest method is using `devtools`:

```
R
setwd("atom2seq")
devtools::install()

setwd("../categoryCompareANY")
devtools::install()
```


For actually running the scripts, you will also need to modify the alias for *interpro* and environment variables *output_dir* and *PDB_dir*, and *R_LIBS* (if using a custom R library path) at the top of mainScripts/run.master.bh. Note that *output_dir* should correspond to *zincPaperAnalysis/output*, which corresponds to *../output* when in *mainScripts*.

Finally, you also need a local copy of the PDB. 

rsync -rlpt -v -z --delete --port=33444 \ > rsync.wwpdb.org::ftp_data/structures/divided/pdb/ ./pdb 

## Running

To run the full analysis:

```
cd mainScripts
./run.master.bh
```
