# creating multiple k-graphs

# There is a graph we need examining cluster distances, jaccard similarity, structural vs functional distances rho 
# and p-values. We want this in a single graphic that makes it easy to compare across num_ligands and metric

library(ggplot2)
library(Cairo)
load("paper_data/measures_over_k.RData")

level_order <- c("jaccard", "max - sumdiff", "rho", "-1 * log10(p)")

# normal clusters

normal_jaccard <- data.frame(cluster = seq(1, 30), value = jac.norm, type = "jaccard", n_lig = "all")
normal_sumdiff <- data.frame(cluster = seq(1, 30), value = max(sumdiff.norm) - sumdiff.norm, type = "max - sumdiff", n_lig = "all")

normal_rho <- data.frame(cluster = rep(seq(3, 30), 4), 
                         value = c(rhop.1.normal$rho,
                                   rhop.2.normal$rho,
                                   rhop.3.normal$rho,
                                   rhop.4.normal$rho),
                         type = "rho",
                         n_lig = rep(c("1", "2", "3", "4"), each = 28))
normal_p <- data.frame(cluster = rep(seq(3, 30), 4), 
                       value = c(rhop.1.normal$p_value,
                                 rhop.2.normal$p_value,
                                 rhop.3.normal$p_value,
                                 rhop.4.normal$p_value),
                       type = "-1 * log10(p)",
                       n_lig = rep(c("1", "2", "3", "4"), each = 28))

min_p <- min(normal_p$value[normal_p$value != 0])
normal_p$value[normal_p$value <= 1e-10] <- 1e-10
normal_p$value <- -1 * log10(normal_p$value)

normal_metrics <- do.call(rbind, list(normal_jaccard, normal_sumdiff, normal_rho, normal_p))
normal_metrics$type <- ordered(normal_metrics$type, levels = level_order)

Cairo(file = "paper_data/normal_metrics.png", type = "png")
ggplot(normal_metrics, aes(x = cluster, y = value, color = n_lig)) + geom_point() + geom_line() + facet_grid(type ~ ., scales = "free_y") + ggtitle("normal")
dev.off()

### compressed clusters
### 
compressed_jaccard <- data.frame(cluster = seq(1, 30), value = jac.comp, type = "jaccard", n_lig = "all")
compressed_sumdiff <- data.frame(cluster = seq(1, 30), value = max(sumdiff.comp) - sumdiff.comp, type = "max - sumdiff", n_lig = "all")

compressed_rho <- data.frame(cluster = rep(seq(3, 30), 4),
                             value = c(rhop.1.comp$rho,
                                       rhop.2.comp$rho,
                                       rhop.3.comp$rho,
                                       rhop.4.comp$rho),
                             type = "rho",
                             n_lig = rep(c("1", "2", "3", "4"), each = 28))

compressed_p <- data.frame(cluster = rep(seq(3, 30), 4),
                           value = c(rhop.1.comp$p_value,
                                     rhop.2.comp$p_value,
                                     rhop.3.comp$p_value,
                                     rhop.4.comp$p_value),
                           type = "-1 * log10(p)",
                           n_lig = rep(c("1", "2", "3", "4"), each = 28))
compressed_p$value <- -1 * log10(compressed_p$value)

compressed_metrics <- do.call(rbind, list(compressed_jaccard, compressed_sumdiff, compressed_rho, compressed_p))
compressed_metrics$type <- ordered(compressed_metrics$type, levels = level_order)

Cairo(file = "paper_data/compressed_metrics.png", type = "png")
ggplot(compressed_metrics, aes(x = cluster, y = value, color = n_lig)) + geom_point() + geom_line() + facet_grid(type ~ ., scales = "free_y") + ggtitle("Compressed")
dev.off()

### combined cluster measures
load("paper_data/measures_over_k_combined.RData")

comb_jaccard <- data.frame(cluster = seq(1, 30), value = jac.comb, type = "jaccard", n_lig = "all")
comb_sumdiff <- data.frame(cluster = seq(1, 30), value = max(sumdiff.comb) - sumdiff.comb, type = "max - sumdiff", n_lig = "all")

comb_rho <- data.frame(cluster = rep(seq(3, 30), 4), 
                         value = c(rhop.1.comb$rho,
                                   rhop.2.comb$rho,
                                   rhop.3.comb$rho,
                                   rhop.4.comb$rho),
                         type = "rho",
                         n_lig = rep(c("1", "2", "3", "4"), each = 28))
comb_p <- data.frame(cluster = rep(seq(3, 30), 4), 
                       value = c(rhop.1.comb$p_value,
                                 rhop.2.comb$p_value,
                                 rhop.3.comb$p_value,
                                 rhop.4.comb$p_value),
                       type = "-1 * log10(p)",
                       n_lig = rep(c("1", "2", "3", "4"), each = 28))

min_p <- min(comb_p$value[comb_p$value != 0])
comb_p$value[comb_p$value <= 1e-10] <- 1e-10
comb_p$value <- -1 * log10(comb_p$value)

comb_metrics <- do.call(rbind, list(comb_jaccard, comb_sumdiff, comb_rho, comb_p))
comb_metrics$type <- ordered(comb_metrics$type, levels = level_order)

Cairo(file = "paper_data/combined_metrics_figS3.png", type = "png")
ggplot(comb_metrics, aes(x = cluster, y = value, color = n_lig)) + geom_point() + geom_line() + facet_grid(type ~ ., scales = "free_y") + ggtitle("combined")
dev.off()
