#!/usr/bin/perl
#
#./zn_protein_nucleicacid-3.pl path_zn_protein_files output_file_appendix
#
# to test how many the zinc binding protein vs. nucleic acid
#
# output files: 1) only_na_or_none_$outfileAppedix.txt" : entries that with less than 20 amino acids as component
#              2) protein_or_both_$outfileAppedix.txt": entries that with at least 20 a.a as component
#
# input: file of paths to pdb files that contain at least one zinc in the HETATM 
#
# the revised version of v3: more versatile output

use strict;

my $zn_protein_file = shift @ARGV;
my $outfileNa = shift @ARGV;
my $outfilePro = shift @ARGV;
open (PATH, $zn_protein_file);
# my @zn_protein_paths = <PATH>;

my %aa_test = ( "GLY" => 1, "CYS" => 1, 
		"ALA" => 1, "HIS" => 1,
		"VAL" => 1, "LYS" => 1,
		"LEU" => 1, "ARG" => 1,
		"ILU" => 1, "GLU" => 1,
		"PRO" => 1, "ASP" => 1,
		"PHE" => 1, "GLN" => 1,
		"TYR" => 1, "ASN" => 1,
		"THR" => 1, "MET" => 1,
		"SER" => 1, "TRP" => 1,);
my %nt_test = ("A" => 1, "U"=> 1, "C"=> 1, "G"=>1, "I" =>1, "DA"=>1, "DC"=>1, "DG"=>1, "DT"=>1, "DI"=>1,);
my ( @both , @protein, @nucleicacid, @neither);
my $count_total;
foreach my $path (<PATH>) 
  {
  chomp $path;
  my $aa_count = 0;
  my $nt_count = 0;
  my $found_seqres = 0;
  open (PDBFILE, "zcat $path|");
  while (my $line = <PDBFILE>)
    {
    chomp $line;
    if ($line =~ /^SEQRES/ )
      {
      $found_seqres = 1;
      foreach my $res (split(/\s+/,substr($line,19)))
	{
        $aa_count += $aa_test{$res};
        $nt_count += $nt_test{$res};
        }  
      }
    elsif($found_seqres)
      { last; }
    }

  if (($aa_count > 19) && ($nt_count > 9)) # definition for having both protein and nucleic acid
    { push (@both, $path) ; }
  elsif ($aa_count > 19) # definition for protein
    { push (@protein, $path) }
  elsif ($nt_count > 9) # definition for nucleic acid
    { push (@nucleicacid, $path) ;}
  else # it is neither
    { push (@neither, $path) ;}
  }

$count_total = @protein + @nucleicacid + @both + @neither;

print "Total entries: $count_total \n";
print "Entries that are both protein and nucleic acid: ", scalar @both, "\n";
print "Entries that are neither protein or na: ", scalar @neither, "\n";
print "Entries that are only nucleic acid: ", scalar @nucleicacid, "\n";
print "Entries that are only protein: ", scalar @protein, "\n";

open ONLYNA, ">$outfileNa";
open PRONA, ">$outfilePro";

print ONLYNA "Entries paths for only nucleic acid: \n", join ("\n", @nucleicacid);
print ONLYNA "\n\n\n\nEntries paths for neither pro or na:\n", join ("\n", @neither);

print PRONA join ("\n", @both);
print PRONA "\n";
print PRONA join ("\n", @protein);

close ONLYNA;
close PRONA;





